@@ -71,7 +71,7 @@ import org.apache.juli.logging.LogFactory;
  * requests.  Requests of any other type will simply be passed through.
  *
  * @author Craig R. McClanahan
- * @version $Revision: 892545 $ $Date: 2009-12-20 02:04:17 +0100 (So, 20. Dez 2009) $
+ * @version $Id: AuthenticatorBase.java 939336 2010-04-29 15:00:41Z kkolinko $
  */
 
 
@@ -98,6 +98,11 @@ public abstract class AuthenticatorBase
     protected static final int SESSION_ID_BYTES = 16;
 
 
+    /**
+     * Default authentication realm name.
+     */
+    protected static final String REALM_NAME = "Authentication required";
+
     /**
      * The message digest algorithm to be used when generating session
      * identifiers.  This must be an algorithm supported by the
