@@ -68,7 +68,6 @@ import org.apache.catalina.core.ApplicationFilterFactory;
 import org.apache.catalina.realm.GenericPrincipal;
 import org.apache.catalina.util.Enumerator;
 import org.apache.catalina.util.ParameterMap;
-import org.apache.catalina.util.RequestUtil;
 import org.apache.catalina.util.StringManager;
 import org.apache.catalina.util.StringParser;
 
@@ -78,7 +77,7 @@ import org.apache.catalina.util.StringParser;
  *
  * @author Remy Maucherat
  * @author Craig R. McClanahan
- * @version $Revision: 892545 $ $Date: 2009-12-20 02:04:17 +0100 (So, 20. Dez 2009) $
+ * @version $Id: Request.java 955665 2010-06-17 16:23:22Z kkolinko $
  */
 
 public class Request
@@ -2252,26 +2251,23 @@ public class Request
             return;
         
         if (response != null) {
-            Cookie newCookie = new Cookie(Globals.SESSION_COOKIE_NAME,
-                    newSessionId);
-            newCookie.setMaxAge(-1);
-            String contextPath = null;
-            if (!response.getConnector().getEmptySessionPath()
-                    && (context != null)) {
-                contextPath = context.getEncodedPath();
+            String scName = null;
+            if (context != null) {
+                scName = context.getSessionCookieName();
             }
-            if ((contextPath != null) && (contextPath.length() > 0)) {
-                newCookie.setPath(contextPath);
-            } else {
-                newCookie.setPath("/");
-            }
-            if (isSecure()) {
-                newCookie.setSecure(true);
+            if (scName == null) {
+                scName = Globals.SESSION_COOKIE_NAME;
             }
+            
+            Cookie newCookie = new Cookie(scName, newSessionId);
+
+            configureSessionCookie(newCookie);
+
             if (context == null) {
-            	response.addCookieInternal(newCookie, false);
+            	response.addSessionCookieInternal(newCookie, false);
             } else {
-            	response.addCookieInternal(newCookie, context.getUseHttpOnly());
+            	response.addSessionCookieInternal(newCookie,
+            	        context.getUseHttpOnly());
             }
         }
     }
@@ -2395,10 +2391,13 @@ public class Request
         // Creating a new session cookie based on that session
         if ((session != null) && (getContext() != null)
                && getContext().getCookies()) {
-            Cookie cookie = new Cookie(Globals.SESSION_COOKIE_NAME,
-                                       session.getIdInternal());
+            String scName = context.getSessionCookieName();
+            if (scName == null) {
+                scName = Globals.SESSION_COOKIE_NAME;
+            }
+            Cookie cookie = new Cookie(scName, session.getIdInternal());
             configureSessionCookie(cookie);
-            response.addCookieInternal(cookie, context.getUseHttpOnly());
+            response.addSessionCookieInternal(cookie, context.getUseHttpOnly());
         }
 
         if (session != null) {
@@ -2417,15 +2416,27 @@ public class Request
      */
     protected void configureSessionCookie(Cookie cookie) {
         cookie.setMaxAge(-1);
+        
+        Context ctxt = getContext();
+        
         String contextPath = null;
-        if (!connector.getEmptySessionPath() && (getContext() != null)) {
-            contextPath = getContext().getEncodedPath();
+        if (ctxt != null && !getConnector().getEmptySessionPath()) {
+            if (ctxt.getSessionCookiePath() != null) {
+                contextPath = ctxt.getSessionCookiePath();
+            } else {
+                contextPath = ctxt.getEncodedPath();
+            }
         }
         if ((contextPath != null) && (contextPath.length() > 0)) {
             cookie.setPath(contextPath);
         } else {
             cookie.setPath("/");
         }
+        
+        if (ctxt != null && ctxt.getSessionCookieDomain() != null) {
+            cookie.setDomain(ctxt.getSessionCookieDomain());
+        }
+
         if (isSecure()) {
             cookie.setSecure(true);
         }
@@ -2583,7 +2594,9 @@ public class Request
                 }
                 return;
             }
-            parameters.processParameters(formData, 0, formData.length);
+            if (formData != null) {
+                parameters.processParameters(formData, 0, formData.length);
+            }
         }
 
     }
@@ -2629,6 +2642,9 @@ public class Request
                 body.append(buffer, 0, len);
             }
         }
+        if (body.getLength() == 0) {
+            return null;
+        }
         if (body.getLength() < body.getBuffer().length) {
             int length = body.getLength();
             byte[] result = new byte[length];
