@@ -63,7 +63,7 @@ import org.apache.catalina.Globals;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 739532 $ $Date: 2009-01-31 10:52:13 +0100 (Sa, 31. Jan 2009) $
+ * @version $Id: ApplicationContext.java 946844 2010-05-21 01:19:48Z kkolinko $
  */
 
 public class ApplicationContext
@@ -492,8 +492,12 @@ public class ApplicationContext
                     return new URL
                         ("jndi", "", 0, getJNDIUri(hostName, fullPath),
                          new DirContextURLStreamHandler(resources));
-                } catch (Exception e) {
+                } catch (NamingException e) {
                     // Ignore
+                } catch (Exception e) {
+                    // Unexpected
+                    log(sm.getString("applicationContext.lookup.error", path,
+                            getContextPath()), e);
                 }
             }
         }
@@ -529,7 +533,12 @@ public class ApplicationContext
                 Object resource = resources.lookup(path);
                 if (resource instanceof Resource)
                     return (((Resource) resource).streamContent());
+            } catch (NamingException e) {
+                // Ignore
             } catch (Exception e) {
+                // Unexpected
+                log(sm.getString("applicationContext.lookup.error", path,
+                        getContextPath()), e);
             }
         }
         return (null);
