@@ -37,7 +37,7 @@ import org.apache.tomcat.jni.Library;
  * 
  * @author Remy Maucherat
  * @author Filip Hanik
- * @version $Revision: 898409 $ $Date: 2010-01-12 17:39:17 +0100 (Di, 12. Jan 2010) $
+ * @version $Id: AprLifecycleListener.java 939336 2010-04-29 15:00:41Z kkolinko $
  * @since 4.1
  */
 
@@ -46,6 +46,7 @@ public class AprLifecycleListener
 
     private static Log log = LogFactory.getLog(AprLifecycleListener.class);
 
+    private static boolean instanceCreated = false;
     /**
      * The string manager for this package.
      */
@@ -71,9 +72,14 @@ public class AprLifecycleListener
     protected static boolean aprAvailable = false;
 
     public static boolean isAprAvailable() {
-        init();
+        //https://issues.apache.org/bugzilla/show_bug.cgi?id=48613
+        if (instanceCreated) init();
         return aprAvailable;
     }
+    
+    public AprLifecycleListener() {
+        instanceCreated = true;
+    }
 
     // ---------------------------------------------- LifecycleListener Methods
 
