@@ -24,12 +24,14 @@ import java.security.Principal;
 import java.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashMap;
-import java.util.HashSet;
 import java.util.Hashtable;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
+import java.util.Map.Entry;
 
 import javax.naming.Context;
 import javax.naming.CommunicationException;
@@ -167,7 +169,7 @@ import org.apache.tomcat.util.buf.CharChunk;
  *
  * @author John Holman
  * @author Craig R. McClanahan
- * @version $Revision: 898707 $ $Date: 2010-01-13 11:37:53 +0100 (Mi, 13. Jan 2010) $
+ * @version $Id: JNDIRealm.java 947616 2010-05-24 12:05:44Z markt $
  */
 
 public class JNDIRealm extends RealmBase {
@@ -1421,10 +1423,10 @@ public class JNDIRealm extends RealmBase {
          if (containerLog.isTraceEnabled()) {
              if (validated) {
                  containerLog.trace(sm.getString("jndiRealm.authenticateSuccess",
-                                  user.username));
+                                  user.getUserName()));
              } else {
                  containerLog.trace(sm.getString("jndiRealm.authenticateFailure",
-                                  user.username));
+                                  user.getUserName()));
              }
          }
          return (validated);
@@ -1450,7 +1452,7 @@ public class JNDIRealm extends RealmBase {
         if (info == null || credentials == null)
             return (false);
 
-        String password = info.password;
+        String password = info.getPassword();
         if (password == null)
             return (false);
 
@@ -1540,7 +1542,7 @@ public class JNDIRealm extends RealmBase {
          if (credentials == null || user == null)
              return (false);
 
-         String dn = user.dn;
+         String dn = user.getDN();
          if (dn == null)
              return (false);
 
@@ -1604,8 +1606,8 @@ public class JNDIRealm extends RealmBase {
         if (user == null)
             return (null);
 
-        String dn = user.dn;
-        String username = user.username;
+        String dn = user.getDN();
+        String username = user.getUserName();
 
         if (dn == null || username == null)
             return (null);
@@ -1614,9 +1616,10 @@ public class JNDIRealm extends RealmBase {
             containerLog.trace("  getRoles(" + dn + ")");
 
         // Start with roles retrieved from the user entry
-        ArrayList<String> list = user.roles;
-        if (list == null) {
-            list = new ArrayList<String>();
+        List<String> list = new ArrayList<String>(); 
+        List<String> userRoles = user.getRoles();
+        if (userRoles != null) {
+            list.addAll(userRoles); 
         }
         if (commonRole != null)
             list.add(commonRole);
@@ -1679,12 +1682,12 @@ public class JNDIRealm extends RealmBase {
             // Directory Groups". It avoids group slurping and handles cyclic group memberships as well.
             // See http://middleware.internet2.edu/dir/ for details
 
-            Set<String> newGroupDNs = new HashSet<String>(groupMap.keySet());
-            while (!newGroupDNs.isEmpty()) {
-                Set<String> newThisRound = new HashSet<String>(); // Stores the groups we find in this iteration
+            Map<String, String> newGroups = new HashMap<String,String>(groupMap);
+            while (!newGroups.isEmpty()) {
+                Map<String, String> newThisRound = new HashMap<String, String>(); // Stores the groups we find in this iteration
 
-                for (String groupDN : newGroupDNs) {
-                    filter = roleFormat.format(new String[] { groupDN });
+                for (Entry<String, String> group : newGroups.entrySet()) {
+                    filter = roleFormat.format(new String[] { group.getKey(), group.getValue() });
 
                     if (containerLog.isTraceEnabled()) {
                         containerLog.trace("Perform a nested group search with base "+ roleBase + " and filter " + filter);
@@ -1702,7 +1705,7 @@ public class JNDIRealm extends RealmBase {
                             String name = getAttributeValue(roleName, attrs);
                             if (name != null && dname != null && !groupMap.keySet().contains(dname)) {
                                 groupMap.put(dname, name);
-                                newThisRound.add(dname);
+                                newThisRound.put(dname, name);
 
                                 if (containerLog.isTraceEnabled()) {
                                     containerLog.trace("  Found nested role " + dname + " -> " + name);
@@ -1716,11 +1719,12 @@ public class JNDIRealm extends RealmBase {
                     }
                 }
 
-                newGroupDNs = newThisRound;
+                newGroups = newThisRound;
             }
         }
 
-        return new ArrayList<String>(groupMap.values());
+        list.addAll(groupMap.values());
+        return list;
     }
 
 
@@ -1925,8 +1929,8 @@ public class JNDIRealm extends RealmBase {
         User user = getUser(context, username);
 
         if (user != null) {
-            return new GenericPrincipal(this, user.username, user.password,
-                    getRoles(context, user));
+            return new GenericPrincipal(this, user.getUserName(),
+                    user.getPassword(), getRoles(context, user));
         }
         
         return null;
@@ -2207,26 +2211,45 @@ public class JNDIRealm extends RealmBase {
     }
 
 
+     // ------------------------------------------------------ Private Classes
+    
+     /**
+      * A protected class representing a User
+      */
+     protected static class User {
+         
+         final private String username;
+         final private String dn;
+         final private String password;
+         final private List<String> roles;
+
+         public User(String username, String dn, String password,
+                 List<String> roles) {
+             this.username = username;
+             this.dn = dn;
+             this.password = password;
+             if (roles == null) {
+                 this.roles = Collections.emptyList();
+             } else {
+                 this.roles = Collections.unmodifiableList(roles);
+             }
+         }
+    
+         public String getUserName() {
+             return username;
+         }
+         
+         public String getDN() {
+             return dn;
+         }
+         
+         public String getPassword() {
+             return password;
+         }
+         
+         public List<String> getRoles() {
+             return roles;
+         }
+     }
 }
 
-// ------------------------------------------------------ Private Classes
-
-/**
- * A private class representing a User
- */
-class User {
-    String username = null;
-    String dn = null;
-    String password = null;
-    ArrayList<String> roles = null;
-
-
-    User(String username, String dn, String password,
-            ArrayList<String> roles) {
-        this.username = username;
-        this.dn = dn;
-        this.password = password;
-        this.roles = roles;
-    }
-
-}
