@@ -73,7 +73,7 @@ import org.apache.naming.resources.ResourceAttributes;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 762909 $ $Date: 2009-04-07 21:12:35 +0200 (Di, 07. Apr 2009) $
+ * @version $Id: DefaultServlet.java 939518 2010-04-30 00:08:58Z kkolinko $
  */
 
 public class DefaultServlet
@@ -121,13 +121,19 @@ public class DefaultServlet
     /**
      * Allow customized directory listing per directory.
      */
-    protected String  localXsltFile = null;
+    protected String localXsltFile = null;
 
 
+    /**
+     * Allow customized directory listing per context.
+     */
+    protected String contextXsltFile = null;
+    
+    
     /**
      * Allow customized directory listing per instance.
      */
-    protected String  globalXsltFile = null;
+    protected String globalXsltFile = null;
 
 
     /**
@@ -242,6 +248,7 @@ public class DefaultServlet
         fileEncoding = getServletConfig().getInitParameter("fileEncoding");
 
         globalXsltFile = getServletConfig().getInitParameter("globalXsltFile");
+        contextXsltFile = getServletConfig().getInitParameter("contextXsltFile");
         localXsltFile = getServletConfig().getInitParameter("localXsltFile");
         readmeFile = getServletConfig().getInitParameter("readmeFile");
 
@@ -686,9 +693,9 @@ public class DefaultServlet
                 // to be ignored by the resource that is including us.
                 // Therefore, the only way we can let the including resource
                 // know is by including warning message in response
-                response.getWriter().write(
+                response.getWriter().write(RequestUtil.filter(
                     sm.getString("defaultServlet.missingResource",
-                    requestUri));
+                    requestUri)));
             }
 
             response.sendError(HttpServletResponse.SC_NOT_FOUND,
@@ -1182,6 +1189,9 @@ public class DefaultServlet
                     trimmed.equalsIgnoreCase(localXsltFile))
                     continue;
 
+                if ((cacheEntry.name + trimmed).equals(contextXsltFile))
+                    continue;
+
                 CacheEntry childCacheEntry =
                     resources.lookupCache(cacheEntry.name + resourceName);
                 if (!childCacheEntry.exists) {
@@ -1478,11 +1488,19 @@ public class DefaultServlet
             } catch (NamingException e) {
                 if (debug > 10)
                     log("localXsltFile '" + localXsltFile + "' not found", e);
-
-                return null;
             }
         }
 
+        if (contextXsltFile != null) {
+            InputStream is =
+                getServletContext().getResourceAsStream(contextXsltFile);
+            if (is != null)
+                return is;
+
+            if (debug > 10)
+                log("contextXsltFile '" + contextXsltFile + "' not found");
+        }
+
         /*  Open and read in file in one fell swoop to reduce chance
          *  chance of leaving handle open.
          */
