@@ -61,7 +61,7 @@ import org.apache.tomcat.util.modeler.Registry;
  * be subclassed to create more sophisticated Manager implementations.
  *
  * @author Craig R. McClanahan
- * @version $Revision: 892545 $ $Date: 2009-12-20 02:04:17 +0100 (So, 20. Dez 2009) $
+ * @version $Id: ManagerBase.java 939518 2010-04-30 00:08:58Z kkolinko $
  */
 
 public abstract class ManagerBase implements Manager, MBeanRegistration {
@@ -180,7 +180,9 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
     // Number of sessions created by this manager
     protected int sessionCounter=0;
 
-    protected int maxActive=0;
+    protected volatile int maxActive=0;
+
+    private final Object maxActiveUpdateLock = new Object();
 
     // number of duplicated session ids - anything >0 means we have problems
     protected int duplicates=0;
@@ -765,7 +767,11 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
         sessions.put(session.getIdInternal(), session);
         int size = sessions.size();
         if( size > maxActive ) {
-            maxActive = size;
+            synchronized(maxActiveUpdateLock) {
+                if( size > maxActive ) {
+                    maxActive = size;
+                }
+            }
         }
     }
 
@@ -1107,7 +1113,9 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
 
 
     public void setMaxActive(int maxActive) {
-        this.maxActive = maxActive;
+        synchronized (maxActiveUpdateLock) {
+            this.maxActive = maxActive;
+        }
     }
 
 
