@@ -59,7 +59,7 @@ import org.apache.catalina.security.SecurityUtil;
  *
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
- * @version $Revision: 612761 $ $Date: 2008-01-17 09:48:45 +0100 (Do, 17. Jan 2008) $
+ * @version $Id: StandardManager.java 942683 2010-05-10 08:39:48Z kfujino $
  */
 
 public class StandardManager
@@ -395,6 +395,12 @@ public class StandardManager
                     session.setManager(this);
                     sessions.put(session.getIdInternal(), session);
                     session.activate();
+                    if (!session.isValidInternal()) {
+                        // If session is already invalid,
+                        // expire session to prevent memory leak.
+                        session.setValid(true);
+                        session.expire();
+                    }
                     sessionCounter++;
                 }
             } catch (ClassNotFoundException e) {
