@@ -78,7 +78,7 @@ import org.apache.catalina.security.SecurityUtil;
  * @author Craig R. McClanahan
  * @author Sean Legassick
  * @author <a href="mailto:jon@latchkey.com">Jon S. Stevens</a>
- * @version $Revision: 900131 $ $Date: 2010-01-17 13:46:53 +0100 (So, 17. Jan 2010) $
+ * @version $Id: StandardSession.java 946841 2010-05-21 00:56:52Z kkolinko $
  */
 
 public class StandardSession
@@ -750,10 +750,15 @@ public class StandardSession
                     manager.setSessionMaxAliveTime(timeAlive);
                 }
                 int numExpired = manager.getExpiredSessions();
-                numExpired++;
-                manager.setExpiredSessions(numExpired);
+                if (numExpired < Integer.MAX_VALUE) {
+                    numExpired++;
+                    manager.setExpiredSessions(numExpired);
+                }
+
                 int average = manager.getSessionAverageAliveTime();
-                average = ((average * (numExpired-1)) + timeAlive)/numExpired;
+                // Using long, as otherwise (average * numExpired) might overflow 
+                average = (int) (((((long) average) * (numExpired - 1)) + timeAlive)
+                        / numExpired);
                 manager.setSessionAverageAliveTime(average);
             }
 
