@@ -29,12 +29,13 @@ import java.net.Socket;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
+import java.util.logging.LogManager;
 
 import org.apache.catalina.Container;
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleException;
-import org.apache.catalina.Server;
 import org.apache.catalina.core.StandardServer;
+import org.apache.juli.ClassLoaderLogManager;
 import org.apache.tomcat.util.digester.Digester;
 import org.apache.tomcat.util.digester.Rule;
 import org.xml.sax.Attributes;
@@ -59,7 +60,7 @@ import org.xml.sax.InputSource;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 892773 $ $Date: 2009-12-21 12:38:21 +0100 (Mo, 21. Dez 2009) $
+ * @version $Id: Catalina.java 946849 2010-05-21 01:38:20Z kkolinko $
  */
 
 public class Catalina extends Embedded {
@@ -81,12 +82,6 @@ public class Catalina extends Embedded {
         Catalina.class.getClassLoader();
 
 
-    /**
-     * The server component we are starting or stopping
-     */
-    protected Server server = null;
-
-
     /**
      * Are we starting a new server?
      */
@@ -151,17 +146,6 @@ public class Catalina extends Embedded {
     }
 
 
-    /**
-     * Set the server instance we are configuring.
-     *
-     * @param server The new server
-     */
-    public void setServer(Server server) {
-
-        this.server = server;
-
-    }
-
     // ----------------------------------------------------------- Main Program
 
     /**
@@ -398,7 +382,7 @@ public class Catalina extends Embedded {
             arguments(arguments);
         }
 
-        if( server == null ) {
+        if( getServer() == null ) {
             // Create and execute our Digester
             Digester digester = createStopDigester();
             digester.setClassLoader(Thread.currentThread().getContextClassLoader());
@@ -419,16 +403,16 @@ public class Catalina extends Embedded {
 
         // Stop the existing server
         try {
-            if (server.getPort()>0) {
-            	String hostAddress = InetAddress.getByName("localhost").getHostAddress();
-            	Socket socket = new Socket(hostAddress, server.getPort());
-            	OutputStream stream = socket.getOutputStream();
-            	String shutdown = server.getShutdown();
-            	for (int i = 0; i < shutdown.length(); i++)
-            		stream.write(shutdown.charAt(i));
-            	stream.flush();
-            	stream.close();
-            	socket.close();
+            if (getServer().getPort()>0) { 
+                String hostAddress = InetAddress.getByName("localhost").getHostAddress();
+                Socket socket = new Socket(hostAddress, getServer().getPort());
+                OutputStream stream = socket.getOutputStream();
+                String shutdown = getServer().getShutdown();
+                for (int i = 0; i < shutdown.length(); i++)
+                    stream.write(shutdown.charAt(i));
+                stream.flush();
+                stream.close();
+                socket.close();
             } else {
                 log.error(sm.getString("catalina.stopServer"));
                 System.exit(1);
@@ -535,11 +519,15 @@ public class Catalina extends Embedded {
         initStreams();
 
         // Start the new server
-        if (server instanceof Lifecycle) {
+        if (getServer() instanceof Lifecycle) {
             try {
-                server.initialize();
+                getServer().initialize();
             } catch (LifecycleException e) {
-                log.error("Catalina.start", e);
+                if (Boolean.getBoolean("org.apache.catalina.startup.EXIT_ON_INIT_FAILURE"))
+                    throw new java.lang.Error(e);
+                else   
+                    log.error("Catalina.start", e);
+                
             }
         }
 
@@ -576,11 +564,11 @@ public class Catalina extends Embedded {
      */
     public void start() {
 
-        if (server == null) {
+        if (getServer() == null) {
             load();
         }
 
-        if (server == null) {
+        if (getServer() == null) {
             log.fatal("Cannot start server. Server instance is not configured.");
             return;
         }
@@ -588,9 +576,9 @@ public class Catalina extends Embedded {
         long t1 = System.nanoTime();
         
         // Start the new server
-        if (server instanceof Lifecycle) {
+        if (getServer() instanceof Lifecycle) {
             try {
-                ((Lifecycle) server).start();
+                ((Lifecycle) getServer()).start();
             } catch (LifecycleException e) {
                 log.error("Catalina.start: ", e);
             }
@@ -607,6 +595,15 @@ public class Catalina extends Embedded {
                     shutdownHook = new CatalinaShutdownHook();
                 }
                 Runtime.getRuntime().addShutdownHook(shutdownHook);
+                
+                // If JULI is being used, disable JULI's shutdown hook since
+                // shutdown hooks run in parallel and log messages may be lost
+                // if JULI's hook completes before the CatalinaShutdownHook()
+                LogManager logManager = LogManager.getLogManager();
+                if (logManager instanceof ClassLoaderLogManager) {
+                    ((ClassLoaderLogManager) logManager).setUseShutdownHook(
+                            false);
+                }
             }
         } catch (Throwable t) {
             // This will fail on JDK 1.2. Ignoring, as Tomcat can run
@@ -631,6 +628,14 @@ public class Catalina extends Embedded {
             // doesn't get invoked twice
             if (useShutdownHook) {
                 Runtime.getRuntime().removeShutdownHook(shutdownHook);
+
+                // If JULI is being used, re-enable JULI's shutdown to ensure
+                // log messages are not lost
+                LogManager logManager = LogManager.getLogManager();
+                if (logManager instanceof ClassLoaderLogManager) {
+                    ((ClassLoaderLogManager) logManager).setUseShutdownHook(
+                            true);
+                }
             }
         } catch (Throwable t) {
             // This will fail on JDK 1.2. Ignoring, as Tomcat can run
@@ -638,9 +643,9 @@ public class Catalina extends Embedded {
         }
 
         // Shut down the server
-        if (server instanceof Lifecycle) {
+        if (getServer() instanceof Lifecycle) {
             try {
-                ((Lifecycle) server).stop();
+                ((Lifecycle) getServer()).stop();
             } catch (LifecycleException e) {
                 log.error("Catalina.stop", e);
             }
@@ -654,7 +659,7 @@ public class Catalina extends Embedded {
      */
     public void await() {
 
-        server.await();
+        getServer().await();
 
     }
 
@@ -682,11 +687,21 @@ public class Catalina extends Embedded {
     protected class CatalinaShutdownHook extends Thread {
 
         public void run() {
-
-            if (server != null) {
-                Catalina.this.stop();
+            try {
+                if (getServer() != null) {
+                    Catalina.this.stop();
+                }
+            } catch (Throwable ex) {
+                log.error(sm.getString("catalina.shutdownHookFail"), ex);
+            } finally {
+                // If JULI is used, shut JULI down *after* the server shuts down
+                // so log messages aren't lost
+                LogManager logManager = LogManager.getLogManager();
+                if (logManager instanceof ClassLoaderLogManager) {
+                    ((ClassLoaderLogManager) logManager).shutdown();
+                }
             }
-            
+
         }
 
     }
