@@ -139,6 +139,21 @@ public abstract class Compiler {
             pageInfo.setTrimDirectiveWhitespaces(JspUtil.booleanValue(jspProperty
                     .isTrimDirectiveWhitespaces()));
         }
+        if (ctxt.isTagFile()) {
+            try {
+                double libraryVersion = Double.parseDouble(ctxt.getTagInfo()
+                        .getTagLibrary().getRequiredVersion());
+                if (libraryVersion < 2.0) {
+                    pageInfo.setIsELIgnored("true", null, errDispatcher, true);
+                }
+                if (libraryVersion < 2.1) {
+                    pageInfo.setDeferredSyntaxAllowedAsLiteral("true", null,
+                            errDispatcher, true);
+                }
+            } catch (NumberFormatException ex) {
+                errDispatcher.jspError(ex);
+            }
+        }
 
         ctxt.checkOutputDir();
         String javaFileName = ctxt.getServletJavaFileName();
@@ -419,6 +434,7 @@ public abstract class Compiler {
             }
             uc.getInputStream().close();
         } catch (Exception e) {
+            log.debug("Problem accessing resource. Treat as outdated.", e);
             return true;
         }
 
@@ -481,6 +497,7 @@ public abstract class Compiler {
                     return true;
                 }
             } catch (Exception e) {
+                log.debug("Problem accessing resource. Treat as outdated.", e);
                 return true;
             }
         }
