@@ -50,11 +50,11 @@ public class JspConfig {
     private ServletContext ctxt;
     private volatile boolean initialized = false;
 
-    private String defaultIsXml = null;		// unspecified
+    private final static String defaultIsXml = null;		// unspecified
     private String defaultIsELIgnored = null;	// unspecified
-    private String defaultIsScriptingInvalid = null;
+    private final static String defaultIsScriptingInvalid = null;
     private String defaultDeferedSyntaxAllowedAsLiteral = null;
-    private String defaultTrimDirectiveWhitespaces = null;
+    private final static String defaultTrimDirectiveWhitespaces = null;
     private JspProperty defaultJspProperty;
 
     public JspConfig(ServletContext ctxt) {
@@ -93,8 +93,12 @@ public class JspConfig {
             if (webApp == null
                     || getVersion(webApp) < 2.4) {
                 defaultIsELIgnored = "true";
+                defaultDeferedSyntaxAllowedAsLiteral = "true";
                 return;
             }
+            if (getVersion(webApp) < 2.5) {
+                defaultDeferedSyntaxAllowedAsLiteral = "true";
+            }
             TreeNode jspConfig = webApp.findChild("jsp-config");
             if (jspConfig == null) {
                 return;
@@ -223,7 +227,8 @@ public class JspConfig {
                     defaultJspProperty = new JspProperty(defaultIsXml,
                             defaultIsELIgnored,
                             defaultIsScriptingInvalid,
-                            null, null, null, defaultDeferedSyntaxAllowedAsLiteral, 
+                            null, null, null,
+                            defaultDeferedSyntaxAllowedAsLiteral, 
                             defaultTrimDirectiveWhitespaces);
                     initialized = true;
                 }
