@@ -46,7 +46,7 @@ import org.apache.naming.NamingEntry;
  * Filesystem Directory Context implementation helper class.
  *
  * @author Remy Maucherat
- * @version $Revision: 769338 $ $Date: 2009-04-28 13:25:01 +0200 (Di, 28. Apr 2009) $
+ * @version $Id: FileDirContext.java 939511 2010-04-29 23:48:20Z kkolinko $
  */
 
 public class FileDirContext extends BaseDirContext {
@@ -884,7 +884,7 @@ public class FileDirContext extends BaseDirContext {
 
 
     /**
-     * This specialized resource implementation avoids opening the IputStream
+     * This specialized resource implementation avoids opening the InputStream
      * to the file right away (which would put a lock on the file).
      */
     protected class FileResource extends Resource {
@@ -924,7 +924,9 @@ public class FileDirContext extends BaseDirContext {
         public InputStream streamContent()
             throws IOException {
             if (binaryContent == null) {
-                inputStream = new FileInputStream(file);
+                FileInputStream fis = new FileInputStream(file);
+                inputStream = fis;
+                return fis;
             }
             return super.streamContent();
         }
